import urllib
from django.utils.module_loading import import_string
from .models import SSOUser
from . import REMOTE_AUTH_TOKEN_URL, SSO_USER_STORAGE
from apiclient import client


class SSOAuthBackend(object):
    def __init__(self):
        SSO_USER_STORAGE_CLZ = import_string(SSO_USER_STORAGE)
        self.storageengine = SSO_USER_STORAGE_CLZ()

    def authenticate(self, request_token, auth_token, **credentials):
        code, user_info = client.send_request(
            REMOTE_AUTH_TOKEN_URL + "?" + urllib.urlencode({"request_token": request_token, "auth_token": auth_token}))
        user = user_info["user"]
        u = SSOUser(**user)
        self.storageengine.save(user["id"], u)
        return u

    def get_user(self, user_id):
        return self.storageengine.find(user_id)



import cPickle
from . import SSO_SETTING_CACHE
from django.core.cache import caches
from django.db import IntegrityError
from .models import SSOUser


class SSOUserStorage(object):
    class Meta:
        abstract = True

    def save(self, userid, ssouser):
        raise NotImplementedError()

    def find(self, userid):
        raise NotImplementedError()

    def remove(self, userid):
        raise NotImplementedError()


class SSOUserCacheStorage(SSOUserStorage):
    def __init__(self):
        self.cache = caches(SSO_SETTING_CACHE)

    def _get_cached_id(self, userid):
        return "sso_user_%s" % userid

    def save(self, userid, ssouser):
        self.cache.set(self._get_cached_id(userid),
                       cPickle.dumps(ssouser), timeout=None)

    def find(self, userid):
        return cPickle.loads(self.cache.get(self._get_cached_id(userid)))

    def remove(self, userid):
        self.cache.delete(self._get_cached_id(userid))


class SSOUserDBStorage(SSOUserStorage):
    def save(self, userid, ssouser):
        try:
            ssouser.save()
        except IntegrityError:
            SSOUser.objects.filter(username=ssouser.username).delete()
            ssouser.save()

    def find(self, userid):
        try:
            return SSOUser.objects.get(pk=userid)
        except SSOUser.DoesNotExist:
            return None

    def remove(self, userid):
        pass



import urllib
import urlparse

from django.http import HttpResponseRedirect
from django.conf import settings
from django.contrib.auth import authenticate, login as auth_login
from django.core.exceptions import PermissionDenied
from django.core.urlresolvers import reverse

from .apiclient import client
from . import REMOTE_REQUEST_TOKEN_URL, REMOTE_SSO_LOGIN_URL


def viewAuth(request):
    request_token = request.GET.get("request_token")
    auth_token = request.GET.get("auth_token")
    redirect_to = request.GET.get("redirect", settings.LOGIN_REDIRECT_URL)

    user = authenticate(request_token=request_token, auth_token=auth_token)
    if user is not None:
        auth_login(request, user)  # create session, write cookies
    else:
        raise PermissionDenied
    return HttpResponseRedirect(redirect_to)


def viewLogin(request):
    _, token_info = client.send_request(REMOTE_REQUEST_TOKEN_URL)
    request_token = token_info["request_token"]

    restserver = settings.SSO_API_AUTH_SETTING["url"]
    url_parts = list(urlparse.urlparse(restserver))
    query = {"api_key": settings.SSO_API_AUTH_SETTING["apikey"],
             "request_token": request_token,
             "next": request.build_absolute_uri(
                 reverse("ssoauth") + "?redirect=%s" % request.GET.get("next", settings.LOGIN_REDIRECT_URL))}
    if hasattr(settings, 'SSO_NO_CONFIRM'):
        query.update({'noconfirm': settings.SSO_NO_CONFIRM})
    url_parts[2] = REMOTE_SSO_LOGIN_URL
    url_parts[4] = urllib.urlencode(query)
    ssoLoginURL = urlparse.urlunparse(url_parts)
    return HttpResponseRedirect(ssoLoginURL)
